package net.lecnam.info.ex3;

import java.util.concurrent.BlockingQueue;
import java.util.logging.Logger;
import net.lecnam.info.ex3.Answer.*;
import net.lecnam.info.ex3.Request.*;

public class Client implements Runnable {

	BlockingQueue<Answer> in;
	BlockingQueue<Request> out;

	Client(BlockingQueue<Answer> in, BlockingQueue<Request> out) {
		this.in = in;
		this.out = out;
	}

	static final Logger logger = Logger.getLogger("server");

	public int size() {
		try {
			Request request = new GetSize();
			out.put(request);
			Answer answer = in.take();
			logger.info("Answer: " + answer);
			switch (answer) {
				case Size(var s) -> {
					return s;
				}
				default -> {
					throw new Error("Invalid answer");
				}
			}
		} catch (InterruptedException ex) {
			throw new Error(ex);
		}
	}

	public String get(int index) {
		try {
			Request request = new GetValue(index);
			out.put(request);
			Answer answer = in.take();
			logger.info("Answer: " + answer);
			switch (answer) {
				case Value(var elt) -> {
					return elt;
				}
				default -> {
					throw new Error("Invalid answer");
				}
			}
		} catch (InterruptedException ex) {
			throw new Error(ex);
		}
	}

	public String set(int index, String element) {
		try {
			Request request = new Set(index, element);
			out.put(request);
			Answer answer = in.take();
			logger.info("Answer: " + answer);
			switch (answer) {
				case OldValue(var elt) -> {
					return elt;
				}
				default -> {
					throw new Error("Invalid answer");
				}
			}
		} catch (InterruptedException ex) {
			throw new Error(ex);
		}
	}

	public String remove(int index) {
		try {
			Request request = new Remove(index);
			out.put(request);
			Answer answer = in.take();
			logger.info("Answer: " + answer);
			switch (answer) {
				case OldValue(var elt) -> {
					return elt;
				}
				default -> {
					throw new Error("Invalid answer");
				}
			}
		} catch (InterruptedException ex) {
			throw new Error(ex);
		}
	}

	public void add(int index, String element) {
		try {
			Request request = new Add(index, element);
			out.put(request);
			Answer answer = in.take();
			logger.info("Answer: " + answer);
			switch (answer) {
				case Done() -> {
					return;
				}
				default -> {
					throw new Error("Invalid answer");
				}
			}
		} catch (InterruptedException ex) {
			throw new Error(ex);
		}
	}

	public void stop() {
		try {
			Request request = new Stop();
			out.put(request);
		} catch (InterruptedException ex) {
			throw new Error(ex);
		}
	}

	@Override
	public void run() {
		int s = size();
		System.out.println("size: " + s);
		for (int i = 0; i < s; i++) {
			String elt = get(i);
			System.out.println(elt);
		}
		set(4, "b14");
		for (int i = 0; i < s; i++) {
			String elt = get(i);
			System.out.println(elt);
		}
		add(2, "c89");
		for (int i = 0; i < s; i++) {
			String elt = get(i);
			System.out.println(elt);
		}
		stop();
	}

}
