package net.lecnam.info.ex3;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.BlockingQueue;
import java.util.logging.Logger;
import net.lecnam.info.ex3.Answer.*;
import net.lecnam.info.ex3.Request.*;

public class Server implements Runnable {
	
	BlockingQueue<Request> in;
	BlockingQueue<Answer> out;
	List<String> l;
	
	static final Logger logger = Logger.getLogger("server");
	
	Server(BlockingQueue<Request> in, BlockingQueue<Answer> out, List<String> l) {
		this.in = in;
		this.out = out;
		this.l = new ArrayList<>(l);
	}
	
	@Override
	public void run() {
		try {
			boolean done = false;
			while (!done) {
				Request request = in.take();
				logger.info("Request: " + request);
				switch (request) {
					case Stop() -> {
						done = true;
					}
					case GetSize() -> {
						int s = l.size();
						Answer answer = new Size(s);
						out.put(answer);
					}
					case GetValue(var index) -> {
						String elt = l.get(index);
						Answer answer = new Value(elt);
						out.put(answer);
					}
					case Set(var index, var element) -> {
						String old = l.set(index, element);
						Answer answer = new OldValue(old);
						out.put(answer);
					}						
					case Add(var index, var element) -> {
						l.add(index, element);
						Answer answer = new Done();
						out.put(answer);						
					}
					case Remove(var index) -> {
						String old = l.remove(index);
						Answer answer = new OldValue(old);
						out.put(answer);					
					}
				}
			}
		} catch (InterruptedException ex) {
			throw new Error(ex);
		}
	}
	
}
